****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createcensortable.sas  
*
* Created (mm/dd/yyyy): 12/28/2017
* Last modified: 05/15/2018
* Version: 1.2
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: 
*   
*  Program inputs:   
*  -DPLOCAL.&RUNID._mstr 
* 
*  Program outputs:                                                                                                                                       
*  -MSOC.&RUNID._censor_cida 
*
*  PARAMETERS:                                                                       
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1     02/22/18      RR		Fixed (QRP-502)Rename to CensDays_Value_Cat
*
*    1.2   	 05/15/18	   AP		Re-organized to allow user specified strata via USERSTRATA parameter
*
***************************************************************************************************;

%macro ms_createcensortable();

	%put =====> MACRO CALLED: ms_createcensortable v1.2;

	%ISDATA(dataset=dplocal.&runid._mstr);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;

		data _censorlevels;
			set userstrata;
			where lowcase(tableID) = "t&type.censor";

			*add censorcat_sort as a class var;
			if index(lowcase(levelvars), 'censdays_value_cat') > 0 then do;
				levelvars = tranwrd(levelvars, "censdays_value_cat", "censorcat_sort censdays_value_cat");
			end;
		run;

		%let all_censorstrat = ;
		proc sql noprint;
			select distinct levelvars into: all_censorstrat separated by ' '
			from _censorlevels;
		quit;

		%nonrep(invar = all_censorstrat, outvar = censorstrat);
		%put &censorstrat.;

		%if %index(&censorstrat., censdays_value_cat) >0 %then %do;
			/*Assign censdays_value_cat*/
			%let censdays_value_cat1 = %sysfunc(tranwrd(&censdays_value_cat., +, -999999999));
			%let numcat= %sysfunc(countw(&censdays_value_cat., ' '));
			%put &censdays_value_cat. &numcat.;

			data dplocal.&runid._mstr (drop=i);
				length censdays_value_cat $20;
				set dplocal.&runid._mstr;

				do i=1 to &numcat.;        
					if timetocensor <= scan(scan("&censdays_value_cat1.",i,' '),2, '-') then do;
				    	censdays_value_cat = scan("&censdays_value_cat.",i,' ');
						Censorcat_sort = i;
				       	leave;
				    end;
				end;
			run;
			
			*create squared table;
			data _censdayscat_square;
				length censdays_value_cat $20;
				numcat = countw("&censdays_value_cat."," ") ;
				do a = 1 to numcat;
					censdays_value_cat = scan("&censdays_value_cat.",a,' ');
					Censorcat_sort = a;
					output;
				end;
				drop a;
			run;
		%end;

		/*Loop through each group. Will only loop 1x if same age stratification across all groups*/

		%do group=1 %to %eval(&cidatablesgroup.);
		
			data _null_;
				set infolder.&COHORTFILE.;
				if _n_ = &Group. then do;
					*Age stratification;
					if missing(agestrat) then do;
						call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
					end;
					else do;
						call symputx("agestrat",agestrat);
					end;

					call symputx("itgroup", cohortgrp);
				end;
			run;

			*if running all groups, grouplist is blank, otherwise only aggregate data for current Group and optional nvrexp group;
			%let grouplist = ;
 			%if %eval(&count_agegroup. >1) %then %do;
				%let grouplist = "&itgroup." "&itgroup._nvrexp"; 
			%end;

			/*Square categorical values*/
			%ms_squaredtableshell(squarevars =&censorstrat.,
								  analysisvars=cens_elig cens_dth cens_dpend cens_qryend cens_episend cens_spec cens_event,
								  groups=&grouplist.,
								  analysis=N);

			*Additional squaring of Censdays_value_cat and event_flag;
			%if %index(&censorstrat., censdays_value_cat) >0 %then %do;
				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select x.*,
						   y.*
					from _mastersquare as x,
					_censdayscat_square as y;
				quit;
			%end;

			%if %index(&censorstrat., event_flag) >0 %then %do;
				data _squared_event_flag;
					event_flag = 'Y'; output;
					event_flag = 'N'; output;
				run;

				proc sql noprint undo_policy=none;
        			create table _mastersquare as
       				select base.*,
              			 com.event_flag
		        	from  _mastersquare as base,
		             	  _squared_event_flag as com;
		        quit;
			%end;

		    data _mstr_forcensor;
			    set dplocal.&runid._mstr(rename=timetocensor=censdays_value in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;)
			        _mastersquare (in=b);
			    if a then do;
			        epi=1;
			    end;
			    else do;
					censdays_value=1;
			        epi=0;    
			    end;
		    run;
	
		%macro createcensor(group=, level=);

			proc means data=_mstr_forcensor nway noprint missing;
				var epi cens_elig cens_dth cens_dpend cens_qryend %if %eval(&type. = 2) %then %do; cens_episend cens_spec cens_event %end; ;
				class group &group.;
				output out=_censorout(drop=_:)  sum(epi)=episodes
												sum(cens_elig)=cens_elig
												sum(cens_dth)=cens_dth
												sum(cens_dpend)=cens_dpend
												sum(cens_qryend)=cens_qryend
												%if %eval(&type. = 2) %then %do;
												sum(cens_episend)=cens_episend
												sum(cens_spec)=cens_spec
												sum(cens_event)=cens_event
												%end; ;
			run;

			*Store patient nums to MSOC;
		    %if %eval(&s.=1) %then %do;
	            data &runid._censor_cida;
		            set _censorout;
		            format level $3.;
		            Level="&level.";
	            run;
	        %end;
	        %else %do;
	            data &runid._censor_cida;
	            	set &RUNID._censor_cida
	                	_censorout(in=a);
		                if a then Level="&level.";
	            run;
	        %end;

			proc datasets nowarn noprint lib=work;
				delete _censorout;
			quit;

		%mend;

		%let levelid = ;
		%let levelvars = ;

		%isdata(dataset=_censorlevels);
		%do s = 1 %to %eval(&nobs.);
			data _null_;
				set _censorlevels;
				if _n_ = &s. then do;
					call symputx('levelid', levelid);
					call symputx('levelvars', levelvars);
				end;
			run;
				
			/*Summarize data */
	   	 	%createcensor(group=group &levelvars., level=&levelid.);
		%end;

		proc sort data=&RUNID._censor_cida;
			by group level &censorstrat.;
		run;

		%if %eval(&group.=1) %then %do;
			data msoc.&RUNID._censor_cida;
				retain group level &censorstrat. episodes cens_elig cens_dth cens_dpend cens_qryend %if %eval(&type. = 2) %then %do; cens_episend cens_spec cens_event %end; ;
				set &RUNID._censor_cida;
				keep group level &censorstrat. episodes cens_elig cens_dth cens_dpend cens_qryend %if %eval(&type. = 2) %then %do; cens_episend cens_spec cens_event %end; ;
			run;
		%end;
		%else %do;
			proc append base=MSOC.&RUNID._censor_cida
				        data=&RUNID._censor_cida(keep=group level &censorstrat. episodes cens_elig
												      cens_dth cens_dpend cens_qryend %if %eval(&type. = 2) %then %do; cens_episend cens_spec cens_event %end;) force;
		     run;
		%end;
	%end;
	%end;

	%put NOTE: ******** END OF MACRO: ms_createcensortable v1.2 ********;

%mend ms_createcensortable;
