****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_processinputfiles.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 11/12/2018
* Version: 2.7
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will process the program input files variables.                                        
*   
*  Program inputs:  
*	- a dataset containing the cohort definition codes
*	- a dataset containing inclusion/exclusion codes
*	- a dataset containing the monitoring file period
*	- a dataset containing the cohort definition parameters
*	- a dataset containing the stratification levels
*             	                        
*  Program outputs:                                                                                                                                       
*	- datasets with formatted input files variables
*   - _diag, _proc, _ndc and _labs that will be used for code extraction
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*     1.3     04/13/15		DM		Added defensive coding to remove true duplicates in the input files
*
*     1.4     04/21/15		EM		Created the looks dataset to have looks with non overlapping dates
*                                   Added defensive coding checks to check surveillance parameters accuracy
*
*	  1.5	  06/26/2015	DM      Added code to ensure HDPS variable integrity. 
*								    Removed the code from 1.4 related to Surveillance activity
*
*	  1.6	  09/14/2015	DM      Added code to process new Type 4 analysis input files
*
*	  1.7	  01/15/2016	VC      PreExisting conditions enhancements
*
*	  1.8	  06/27/2016	DM		Added the code removed in 1.5 related to Surveillance activity
*
*	  1.9	  11/30/2016 	AP		Added code to adjust EndDate if DP Max Date < EndDate
*
*	  2.0	  03/23/2017 	DM		Changes for the QRP 4.0 version
*
*	  2.1     05/05/2017    AP		Added code to compile list of groups to output race/hispanic info
*
*	  2.2	  06/26/2017	AP		Modified for the additional of Type 5 analysis
*									Updated codetype length in ComborbsCodes to avoid mismatched length warnings
*									Hardcoded length of code to be the same across all files (QRP-294)
*									Added _aniv and _dte output tables for cohorts identified by age or calendar date
*
*	  2.3	  10/03/2017	AP		Changed PregCodes to PregDur (QRP-388)
*
*	  2.4	  12/21/2017	AP		Implemented CODEDAYS parameter in inclusion/exclusion and covariate files (QRP-455)
*
*	  2.5	  05/15/2018	AP		Processed SWITCHINCLUSIONCODES file and added Type 6 info, 
*									Removed race/hispanic vars
*									Added USERSTRATA input file
*
*	  2.6	  10/20/2018	AP		Modifications for added MIL table and Type 4 PSA analysis 
*
*	  2.7	  11/12/2018	AP		Added code for counting number of enrollment extractions needed
*
***************************************************************************************************;

%macro ms_processInputfiles();

	%put =====> MACRO CALLED: ms_processinputfiles v2.7;

	/**********************************************/
	/* Diagnosis, Procedure, and Dispensing codes */
	/**********************************************/

	*Cohortcodes;	
		data &COHORTCODES.;
		set infolder.&COHORTCODES.;
	   	code=compress(code,'. ');
		run;	

	*InclusionCodes;
	 %if %sysfunc(exist(infolder.&INCLUSIONCODES.))=1 %then %do;
		data &INCLUSIONCODES.;
			set infolder.&INCLUSIONCODES.;
			if CondInclusion=1 then do;
				T1_INDEX="INC";
			    T2_INDEX="INC";
			    T3_INDEX="INC";
				T4_INDEX="INC";
				T5_INDEX="INC";
				T6_INDEX="INC";
			end;
			if CondInclusion=0 then do;
				T1_INDEX="EXC";
			    T2_INDEX="EXC";
			    T3_INDEX="EXC";
				T4_INDEX="EXC";
				T5_INDEX="EXC";
				T6_INDEX="EXC";
			end;

			T2_FUP="NOT";
			T3_COMP="NOT";	
			T3_RISK="NOT";
			T4_FUP="NOT";
			CONC_FUP="NOT";

			*Defensive coding;
			code=compress(code,'. ');
			condlevel=upcase(condlevel);
			subcondlevel=upcase(subcondlevel);
			if missing(codedays) then codedays = 1;
			if missing(excludesupply) then excludesupply = 'N';
		run;

		*Ensure only 1 codedays value per group condlevel subcondlevel;
		proc sql noprint;
			create table _codedays_check as 
			select distinct group
				 , condlevel
				 , subcondlevel
			from &INCLUSIONCODES.
			group by group, condlevel, subcondlevel
			having count(distinct codedays) > 1;
		quit;

		%isdata(dataset=_codedays_check);
		%if %eval(&nobs.>0) %then %do;
			%put WARNING: Different CODEDAYS value specified within the same SUBCOND. Check Inclusion/Exclusion code file integrity;
		%end;

		proc sort data=&INCLUSIONCODES. out=&INCLUSIONCODES.;
			by group CondLevel SubCondLevel;
		run;

		*Create numeric Condlevel and SubCondlevel variables for looping;
		data &INCLUSIONCODES. (drop=l_subcond l_cond);
			set &INCLUSIONCODES.;
			format cond subcond 8.;
			by group CondLevel;

			l_cond=lag(CondLevel);
			l_subcond=lag(SubCondLevel);

			if first.group then do;
				cond=1;
				call missing(l_cond);
			end;
			else do;
				if CondLevel=l_cond then do;
					cond=cond;
				end;
				else if CondLevel^=l_cond then do;
					cond=cond+1;
				end;
			end;
			*Create numeric subcondlevel variable;
			if first.CondLevel then do;
				subcond=1;
				call missing(l_subcond);
			end;
			else do;
				if SubCondLevel=l_subcond then do;
					subcond=subcond;
				end;
				else if SubCondLevel^=l_subcond then do;
					subcond=subcond+1;
				end;
			end;
			retain cond subcond;
		run;

		*Append inclusion and exclusion codes to work.&COHORTCODES.;
		data &COHORTCODES.;	
			set &COHORTCODES.
		        &INCLUSIONCODES.;
		run;
	%end;
	   
	/***********************************************/
	/* TYPE4 Pregnancy Pre/Post term codes         */
	/***********************************************/
	%IF %SYSFUNC(exist(infolder.&PREGDUR.))=1 %THEN %DO;
		data &PREGDUR.;
			set infolder.&PREGDUR.;
			Code=compress(Code,'. ');
			T1_INDEX="NOT";
	        T2_INDEX="NOT";
	        T3_INDEX="NOT";
			T4_INDEX="PCY";
			T5_INDEX="NOT";
			T6_INDEX="NOT";
			T2_FUP="NOT";
		    T3_COMP="NOT";	
		    T3_RISK="NOT";
			T4_FUP="NOT";
			CONC_FUP="NOT";
		run;

		* Make sure pre/post term codes with the same priorities have the same duration;
		proc sql noprint;	
			create table _DurationFreq as 
			select count(distinct Duration) as NumDuration, PriorityGroup1, PriorityGroup2, Priority
			from &PREGDUR.
			group by PriorityGroup1, PriorityGroup2, Priority;

			select max(NumDuration) into :NumDuration 
			from _DurationFreq;
		quit;
			
		%if (&NumDuration. gt 1) %then %do;
		   	%put WARNING: Some PriorityGroup1 or PriorityGroup2 codes have the same priority but different duration;	     
		%end;

		data &COHORTCODES.;
		   set &COHORTCODES.
		       &PREGDUR.;	    
		run;
	%END;

	/***********************************************/
	/* TYPE4 Medication of interest/drug use codes */
	/***********************************************/	
	proc sort nodupkey data=infolder.&COHORTCODES. out=MPNList;
		by T4_INDEX;
		where T4_INDEX =:"MP";
	run;

	%ISDATA(dataset=MPNList);	
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		proc sql noprint;
			select count(T4_INDEX) into :NUM_MPN 
			from MPNList;
		quit;
	%END;
	%put &NUM_MPN.;

	*Defensive coding - remove input files true duplicates;
	proc sort nodupkey data=&COHORTCODES.;
		by _all_;
	run; 

	* Make sure CodeSupply is the same within a particular group;
	proc sql noprint;	
		create table _CodeSupplyFreq as 
		select count(distinct CodeSupply) as NumCodeSupply, group
		from &COHORTCODES.
		group by Group;

		select max(NumCodeSupply) into :NumCodeSupply 
		from _CodeSupplyFreq;
	quit;
		
	%if (&NumCodeSupply. gt 1) %then %do;
	   	%put WARNING: Some group(s) in the &COHORTCODES. input file have more than one value specified for the CodeSupply variable;	     
	%end;

	/*******************/
	/* Monitoring File */
	/*******************/
	%IF %SYSFUNC(exist(infolder.&Monitoringfile.))=1 %THEN %DO;

		*limiting to periodid set by user;
	    data _null_;
			%IF %STR(&PERIODIDSTART.) = %STR() or %LENGTH(&PERIODIDSTART.) = 0. %THEN %DO;
				call symputx("PERIODIDSTART",1);
			%END;
			%IF %STR(&PERIODIDEND.) = %STR() or %LENGTH(&PERIODIDEND.) = 0. %THEN %DO;
				call symputx("PERIODIDEND",99999);
			%END;        
		run;

		*Taking min startfollowup and max enddate;
	    proc means data=infolder.&Monitoringfile. nway noprint;
	        var startfollowup enddate ;
	        where &PERIODIDSTART. <= PeriodId  <= &PERIODIDEND.;
	        output out=&Monitoringfile._agg min(startfollowup)=startfollowup
	                                   	    max(enddate)=enddate;
	    run; 

		data &Monitoringfile.; 
			set infolder.&Monitoringfile.;
		run;

		%macro processdpmaxdate(file);
			data _null_;
				set infolder.&file.;
					if _n_ = 1 then do; /*Only 1 Censor_dpend value allowed*/
		           		call symputx('Censor_DPEnd',upcase(strip(Censor_DPEnd)));  
					end;
				run;

			/*Turn off CENSOR_DPEND if using SURVEILLANCEMODE*/
			%IF "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %THEN %DO;
				%if &Censor_DPEnd. = Y %then %do;
					%let Censor_DPEnd = N;
					%put WARNING: Censor_DPEnd turned off because Surveillance mode is being used;
				%end;
			%end;			

			%if &Censor_DPEnd. = Y %then %do;
				/*add dp_maxdate to monitoring file*/
				data &Monitoringfile._agg(drop=dp_maxdate);
					set &Monitoringfile._agg;
					format dp_maxdate date9.;
					dp_maxdate = &DP_MaxDate.;

					if dp_maxdate < enddate then do;
						enddate = dp_maxdate; /*recode enddate*/
						call symputx("DPMax_Adjust", 1); /*Set DPMax_adjust flag to track adjustment*/

						if dp_maxdate < startfollowup then do;
							enddate = startfollowup; /*recode enddate to beginning of query period*/
							call symputx("DPMax_check", 1); /*Set check var to output warning*/
						end;
					end;	
				run;

				data &Monitoringfile.(drop=dp_maxdate);
					set &Monitoringfile.;
					format dp_maxdate date9.;
					dp_maxdate = &DP_MaxDate.;

					if dp_maxdate < enddate then do;
						enddate = dp_maxdate; /*recode enddate*/
						if dp_maxdate < startfollowup then do;
							enddate = startfollowup; /*recode enddate to beginning of query period*/
						end;
					end;
				run;

				%if %eval(&DPMax_check.=1) %then %do;
					%put WARNING: DP_MAXDATE is prior to query start date. Query End Date and Query End date are set to the same date to avoid downstream warnings.;
					%put Query results should not be used;
				%end;
			%end;
		%mend;

		%IF %SYSFUNC(exist(infolder.&TYPE1FILE.))=1 %THEN %DO;
			%processdpmaxdate(&TYPE1FILE.);
		%end;
		%IF %SYSFUNC(exist(infolder.&TYPE2FILE.))=1 %THEN %DO;
			%processdpmaxdate(&TYPE2FILE.);
		%end;
		%IF %SYSFUNC(exist(infolder.&TYPE5FILE.))=1 %THEN %DO;
			%processdpmaxdate(&TYPE5FILE.);
		%end;
		%IF %SYSFUNC(exist(infolder.&TYPE6FILE.))=1 %THEN %DO;
			%processdpmaxdate(&TYPE6FILE.);
		%end;

		data &Monitoringfile._agg;
	        set &Monitoringfile._agg;    
	        call symputx("startfollowup",startfollowup);
	        call symputx("enddate",enddate);    
	    run;
		%put &startfollowup. &enddate. &numperiodid.;

		* Create Non Overlapping Looks dates;
	    proc sort data=&Monitoringfile. out=looks;
	        by PERIODID;
	    run;

	    data looks;
	        set looks;
	        format lenddate LookStart LookEnd date9.;
	        LookStart=startfollowup;
	        LookEnd=enddate;
	        if periodid>1 then LookStart=lenddate+1;
	        lenddate=enddate;
	        retain lenddate;
	        rename PeriodId = Look;
	        keep PeriodId LookStart LookEnd;
		run; 

	    data _null_;
		    format startfollowup enddate date9.;
		    startfollowup=intnx('month',&startfollowup.,0,'begin');  *in case not at the start of month;
		    enddate=intnx('month',&enddate.,1,'begin')-1; *in case not at the end of month;
		    nummonths=intck('month',startfollowup,enddate)+1;
		    numyears=intck('year',startfollowup,enddate)+1;

		    call symputx("StartYear",year(startfollowup));
		    call symputx("StartMonth",month(startfollowup));
		    call symputx("EndYear",year(enddate));
		    call symputx("EndMonth",month(enddate));
		    call symputx("nummonths",nummonths);
		    call symputx("numyears",numyears);
		run;
    	%PUT &StartYear. &StartMonth. &EndYear. &EndMonth. &nummonths. &numyears.;

		*PER and YEAR macro vars are global;
		%do int = 1 %to %eval(&nummonths.+1);
			%global per&int.;
		%end;
		%do int = 1 %to %eval(&numyears.+1);
			%global year&int.;
		%end;

	    data _null_;
		    do i=1 to &nummonths.+1;
		        call symput("PER"||strip(put(i,best.)),put(intnx('month',&startfollowup.,i-1,'begin'),best.));
		    end;
		    do i=1 to &numyears.+1;
		        call symput("YEAR"||strip(put(i,best.)),put(intnx('year',&startfollowup.,i-1,'begin'),best.));
	    	end;
    	run;

	%END;

	/********************************/
	/* Surveillance mode processing */
	/********************************/

	%IF "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %THEN %DO;
	  	%if &PERIODIDSTART. ne &PERIODIDEND. %then %do;
	  		%put WARNING: Surveillance mode runs for only one period, i.e. PERIODIDSTART must be equal to PERIODIDEND.;
	 	    %abort;
	  	%end;

	    * Prior data existence for Surveillance;
	    %IF %EVAL(&PERIODIDSTART.>=2) %THEN %DO;
	        %let DPLPrior = %ms_check_path(&DPLPrior);
	        %put &DPLPrior.;

	        libname DPLPrior "&DPLPrior." access=readonly;

	      	%ISDATA(dataset=DPLPRIOR.&RUNID._mstr);	
	        %put &NOBS.;
	        %IF %EVAL(&NOBS.=0) %THEN %DO;
	    		%put WARNING: DPLPRIOR.&RUNID._mstr not found or is empty.;
	 	        %abort;
	        %end;

	      	%ISDATA(dataset=DPLPRIOR.&RUNID._signature);	
	        %put &NOBS.;
	        %IF %EVAL(&NOBS.>0) %THEN %DO;
	          data _null_;
	          set DPLPRIOR.&RUNID._signature;
	          if var="PERIODIDEND" then call symput("LASTRUN",strip(VALUE)); 
	          run;
	          %put &LASTRUN.;
	          %if %eval(&PERIODIDSTART.-1 ne &LASTRUN.) %then %do;
	      			%put WARNING: DPLPrior is not pointing to the last run.;
	            %abort;
	          %end;
	        %end;

	        %if %str(%UPCASE("&ANALYSIS."))=%str("PS") %then %do;
	        	proc sql noprint;	
	      	  	  select max(comp_order) 
	      	  	  into :max_comp_order
	      	  	  from infolder.&COMPARISON.;
	          quit;

	          %do k=1 %to &Max_comp_order.;
	          	%ISDATA(dataset=DPLPRIOR.&runID._matched_&k._%eval(&PERIODIDSTART.-1));	
	            %IF %EVAL(&NOBS.=0) %THEN %DO;
	        		%put WARNING: DPLPRIOR.&runID._matched_&k._%eval(&PERIODIDSTART.-1)) not found or is empty.;
	              	%abort;
	            %end;
	          %end; 
	    	%end;
	 	%end;  /*PERIODIDSTART>=2*/
	%end;

	/*********************************/
	/* Preprare Raw extraction files */
	/*********************************/
	%ISDATA(dataset=infolder.&Comorbfile.);	    
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		*Adapt for different lengths to avoid w a r n i n g s later;
	    data ComborbsCodes;
	        length codetype $3. code $11. CODECAT $2.;
	        format code $11.;
	        set infolder.Comorbcodes;
	        if code_grp="DIAG" then CODECAT="DX";
	        Comorbcode=1;
	        output;
	        if wildcard="Y" then do;
	            if Length(code)=3 then do;
	                code=strip(code)||"*";
	                output;
	            end;
	            if Length(code)<=4 then do;
	                code=strip(code)||"*";
	                output;
	            end;
	        end;
		run;
	    %let ComborbsCodes=ComborbsCodes(keep=Code CodeCat Codetype Comorbcode);
	%END;

	%ISDATA(dataset=infolder.&COVARIATECODES.);	    
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		*Select covariates codes (not combination of covatiates);
		*to prevent w a r n i n g when covariatecodes contains >11 length for CC codes, reset length of code*;
		data _&COVARIATECODES.
			CovarCodes;

			length code $11;
			format code $11.;
			set infolder.&COVARIATECODES.(rename=code=code1);
			where codecat not in ("CC");
			code=code1;
			drop code1;
			comorbcode=0;

			/*Defensive if blank*/
			if missing(codedays) then codedays = 1;
			if missing(excludesupply) then excludesupply = 'N';
			excludesupply=upcase(excludesupply);
	 	run;

		/*codedays variable should be specified at the covariate level. Run a check to ensure value is unique within covariate*/
		%let cov_codedays_check = ;
		proc sql noprint;
			select max(count) into: cov_codedays_check
			from
				(select covarnum, count(distinct codedays) as count
				from _&COVARIATECODES.
				group by covarnum)
			group by count;
		quit; 
		%if %eval(&cov_codedays_check.) >1 %then %do;
			%put WARNING: Multiple values of codedays per Covarnum. Revise input file;
		%end; 

		*Count Covars and ComboCovars;
		proc sql noprint;	
		    select count (distinct CovarNum) 
		    into :NumCovars
		    from infolder.&COVARIATECODES.
			where codecat not in ("CC");
		quit;
		proc sql noprint;	
		    select count (distinct CovarNum) 
		    into :NumComb
		    from infolder.&COVARIATECODES.
			where codecat in ("CC");
		quit;
		%put Number of covars: &NumCovars;
		%put Number of covarComb: &NumComb;
	%END;
	%ELSE %DO;
		data CovarCodes;
			studyname = "";
			StockGroup="";
	        Code = "";
			CodeCat = "";
			Codetype = "";
			CARESETTINGPRINCIPAL = "";
			CovarNum = 0; 
			Comorbcode = 0;
			CovFrom=.;
			CovTo=.;
			codedays=.;
			codesupply=.;
			excludesupply='';
		run;

		/*check consistency with PROFILE parameter*/
		%if "&profile" = "y" %then %do;
			%put WARNING: PROFILE set to Y, however covariatecodes not specified. PROFILE will be set to N.;
			%let profile = n;
		%end;
	%END;

	%let CovarCodes=CovarCodes(keep=Code CodeCat Codetype CARESETTINGPRINCIPAL studyname StockGroup CovarNum Comorbcode covfrom covto codedays codesupply excludesupply);
	%let CovarCodes2=if CodeCat in("PX","DX") then output CovMeds;    

	proc sort nodupkey data=CovarCodes out=StudyNames(keep=studyname CovarNum);
	    by CovarNum;
	run;

	*Combine COHORTCODES, ComborbsCodes and CovarCodes codes to pull data from CDM only once;
	data _diag(drop=comb)
	     _proc(drop=comb)
	     _ndc
		 _dte
		 _aniv
		 _mil;

	    set &COHORTCODES.
			&ComborbsCodes.
	        &CovarCodes.;
	    Code=compress(Code,'. ');
	    length=length(code);
	    CodeCat=upcase(CodeCat);
	    comb=0;
	    if CodeCat in('RX') then output _ndc;
	    if CodeCat in('DX') then output _diag;
	    if CodeCat in('PX') then output _proc;
		if codecat in('DT') then output _dte;
		if codecat in('AN') then output _aniv;
		if codecat in('MI') then output _mil;

	    keep Code CodeCat Codetype comb;
	run;

	*Checking for the presence of optional lab vars;
	data _null_;
	    dset=open("&COHORTCODES.");
	    call symput('Labvars',put(varnum(dset,'RawLabResult'),best.));
	run;
	%put &Labvars.;

	%IF %EVAL(&Labvars.>=1) %THEN %DO;
		data _lab;
	        set &COHORTCODES.;
	        Code=compress(Code,'. ');
	        length=length(code);
	        CodeCat=upcase(CodeCat);
	        RawCaresetting=Caresettingprincipal;
	        RawGroup=Group;
	        if CodeCat in('LB') then output _lab;
	        keep Code CodeCat Codetype RawLabResult RawLabDateType RawCaresetting RawGroup codesupply excludesupply;* RawCaresetting;
	    run;
	%END;
	%ELSE %DO;
		data _lab;
	        set &COHORTCODES.(obs=0);
	        RawGroup=Group;
	        keep Code CodeCat Codetype RawGroup codesupply excludesupply;* RawLabResult RawLabDateType Caresettingprincipal;* RawCaresetting;
	    run;
	%END;

	/***************************************************/
	/* Creating the number of groups to create cohorts */
	/***************************************************/
	%isdata(dataset=infolder.&COHORTFILE.); 
	data _null_;
    call symputx("Ngroup","&nobs.");
    run;
	%put The number of scenarios is &ngroup.;


	*To prevent unecessary looping, separate count of groups with different age strat;
	proc sql noprint;
		select count (distinct agestrat) into: count_agegroup
		from infolder.&cohortfile.;
	quit;

 	%if %eval(&count_agegroup. >1) %then %do;
		%let cidatablesgroup = &ngroup.;
	%end;

	*Determine distinct number of enrollment/demographic/death cohorts;
	%macro adddeathtofile(file=, var=);
		%if %sysfunc(exist(infolder.&&&file.))=1 %then %do;
			proc sql noprint;
				create table _coverage as
				select x.*,
					   &var. as censor_dth
				from infolder.&cohortfile. as x,
				infolder.&&&file. as y
				where lowcase(x.cohortgrp) = lowcase(y.group);
			quit;
		%end;
	%mend;

	%adddeathtofile(file=TYPE1FILE, var=y.censor_dth);
	%adddeathtofile(file=TYPE2FILE, var=y.censor_dth);
	%adddeathtofile(file=TYPE3FILE, var=y.t3censor_dth);
   	%adddeathtofile(file=TYPE4FILE, var='N');
	%adddeathtofile(file=TYPE5FILE, var=y.censor_dth);
	%adddeathtofile(file=TYPE6FILE, var=y.censor_dth);

	proc sort data=_coverage out=_coveragenodup nodupkey;
		by coverage enrolgap chartres sex race hispanic censor_dth;
	run;

	data _coveragenodup;
		set _coveragenodup;
		enrollmentnum = _n_;
	run;

	proc sql noprint;
		create table cohortfile_for_enr as
		select x.*,
			   y.enrollmentnum
		from _coverage as x,
		_coveragenodup as y
		where x.coverage = y.coverage and x.enrolgap = y.enrolgap and x.chartres = y.chartres
		   and x.sex = y.sex and x.race = y.race and x.hispanic = y.hispanic
			and x.censor_dth = y.censor_dth;
	quit;

	/******************************************************************/
	/*Determine if the chart variable exists in MSDD enrollment table */  
	/******************************************************************/
	data _null_;
	    dset=open("indata.&ENRTABLE.");
	    call symput ('chartvar',put(varnum(dset,'CHART'),best.));
	run;
	%put &chartvar.;
		
	/*******************/
	/* Stratifications */
	/*******************/

	/*ZIPFILE*/
	%ISDATA(dataset=infolder.&ZIPFILE.);	
	%if %eval(&nobs.>0) %then %do;
		%let geog = Y;
	%end;

	/*USERSTRATA*/
	%ISDATA(dataset=infolder.&USERSTRATA.);	
	%if %eval(&nobs.>0) %then %do;

		data userstrata;
			format levelvars $150.; /*defensive b/c additional vars added by the code*/
			set infolder.&USERSTRATA.;

			levelvars = lowcase(levelvars);
			tableid = lowcase(tableID);

			*add agegroupnum to strata that include age group;
			if index(lowcase(levelvars), 'agegroup') > 0 then do;
				levelvars = tranwrd(levelvars, "agegroup", "agegroup agegroupnum");
			end;

			*add year when month is requested;
			if index(lowcase(levelvars), 'month') > 0 & index(lowcase(levelvars), 'year') = 0 then do;
				levelvars = tranwrd(levelvars, "month", "year month");
			end;

			*defensive: convert * to space (i.e. agegroup*sex to agegroup sex);
			levelvars = tranwrd(levelvars, "*", " ");
		run;

		*Master list of all strata;
		proc sql noprint;
			select distinct lowcase(levelvars) into: allstrat separated by ' '
			from userstrata;
			select distinct lowcase(tableID) into: tables separated by ' '
			from userstrata;
		quit;
		%put &allstrat.;
		%put Tables to output: &tables.;

		*determine if race, hispanic, and/or geog vars are output;
		%if %index(&allstrat., race) > 0 %then %do;
			%let race_out = Y;
		%end;
		%if %index(&allstrat., hispanic) > 0 %then %do;
			%let hispanic_out = Y;
		%end;
		%if %index(&allstrat., zip3) > 0 | %index(&allstrat., state) > 0 | %index(&allstrat., cb_reg) > 0 |
		  %index(&allstrat., hhs_reg) > 0 | %index(&allstrat., zip_uncertain) > 0 %then %do;
			%let geog_out = Y;
		%end;
		%put race = &race_out, hispanic = &hispanic_out., geography= &geog_out.;

		*Integrity check - if geographic strata requested, zipfile must also be specified;
		%if "&geog_out" = "Y" & "&geog" ne "Y" %then %do;
			%put WARNING: Geographic stratifications requested, but ZIPFILE not specified.;
		%end;

		*Determine if censor table should be produced;
		%if %index(&tables., t1censor) > 0 | %index(&tables., t2censor) > 0 %then %do;
			%let censor_output = Y;
		%end;

		*type 4: determine if control cohort should be produced;
		%if %index(&tables., t4nopreg) > 0 %then %do;
			%let t4_nopreggroup = Y;
		%end;

	%end;
	%else %do;
		*USERSTRATA not specified;
		%if "&analysis" = "" %then %do;
		%put WARNING: No stratifications requested. Minimal output will be produced.;
		%end;

		data userstrata;
			tableID = '';
			levelvars = '';
			stop;
		run;
	%end;

	proc datasets nowarn noprint lib=work;
		delete _tmpcensor _tmpuserstrata;
	quit;

	/***************************************************************/
	/* Never Exposed Cohort: Utilization File and Comorbidity File */
	/***************************************************************/

	%ISDATA(dataset=infolder.&TYPE2FILE.);	
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		proc sql;
			create table _cohorttype as
			select a.*, b.NEVEREXPOSEDCOHORT
			from cohortfile_for_enr a, infolder.&TYPE2FILE. b
			where a.cohortgrp = b.group;
		quit;

		data cohortfile (drop = NEVEREXPOSEDCOHORT);
			length cohortgrp $40;
			set _cohorttype;

			output;
			if upcase(strip(NEVEREXPOSEDCOHORT)) = "Y" then do;
				cohortgrp = cats(cohortgrp,"_nvrexp");
				output;
			end;
		run;
	%END;
	%ELSE %DO;
		data cohortfile;
			length cohortgrp $40;
		 	set cohortfile_for_enr;
		run;
	%END;

	%macro createnewinput(indata=, outdata=);
		%ISDATA(dataset=infolder.&indata.);	
	  	%IF %EVAL(&NOBS.>0) %THEN %DO;
			%ISDATA(dataset=infolder.&TYPE2FILE.);	
	  		%IF %EVAL(&NOBS.>0) %THEN %DO;
		  	proc sql;
		  	  create table _&outdata.type as
			  select a.*, b.NEVEREXPOSEDCOHORT
			  from infolder.&indata. a
				left join infolder.&TYPE2FILE. b
			  on upcase(strip(a.group)) = upcase(strip(b.group));
		   	quit;
		   	data &outdata. (drop = NEVEREXPOSEDCOHORT);
				 length group $40;
				set  _&outdata.type;
				 output;
				 if upcase(strip(NEVEREXPOSEDCOHORT)) = "Y" then do;
				 	group = cats(group,"_nvrexp");
					output;
				 end;
		   	run;
		   %END;
		   %ELSE %DO;
			data &outdata.;
				length group $40;
				 set infolder.&indata.;
			run;
		   %END;
		%END;
	%mend createnewinput;
	%createnewinput(indata = &UTILFILE., outdata = utilfile);
	%createnewinput(indata = &COMORBFILE., outdata = COMORBFILE);


	/******************************/
	/* Secondary Episode Analysis */
	/******************************/

	%ISDATA(dataset=infolder.&MULTEVENTFILE.);	
	%if %eval(&nobs.>0) %then %do;
		%let multievent = 1;

		data secondaryinputfile;
			set infolder.&MULTEVENTFILE.;
		run;

		*Adherence;
		%isdata(dataset=infolder.&MULTEVENTFILE_ADHERE.);
		%if %eval(&nobs.) > 0 %then %do;
			%let adherence = 1;
			data adherencefile;
				set infolder.&MULTEVENTFILE_ADHERE.;
			run;
		%end;
	%end;

	%ISDATA(dataset=infolder.&OVERLAPFILE.);	
	%if %eval(&nobs.>0) %then %do;
		%let overlap = 1;

		data secondaryinputfile;
			set infolder.&OVERLAPFILE.;
		run;

		*Adherence;
		%isdata(dataset=infolder.&OVERLAPFILE_ADHERE.);
		%if %eval(&nobs.) > 0 %then %do;
			%let adherence = 1;
			data adherencefile;
				set infolder.&OVERLAPFILE_ADHERE.;
			run;
		%end;
	%end;

	%if %eval(&multievent.+&overlap.>1) %then %do;
		%put WARNING: More than 1 analysis (multiple events and overlap) is requested in the same Run. Only 1 analysis should be requested;
	%end;

	*List of primary episode groups;
	%if %eval(&multievent.=1 | &overlap.=1) %then %do;
		proc sql noprint;
			select distinct cats("'",primary,"'") into: primarygrouplist separated by ', '
			from secondaryinputfile;
		quit;
	%end;

	/********************************/
	/* Concomitant Episode Analysis */
	/********************************/
	%ISDATA(dataset=infolder.&CONCFILE.);	
	%if %eval(&nobs.>0) %then %do;
		%let concomitant = 1;
	%end;


	/**********************************/
	/* Mother-Infant Linkage Analysis */
	/**********************************/
	%ISDATA(dataset=infolder.&MICOHORTFILE.);	
	%if %eval(&nobs.>0) %then %do;
		%let milinkage = 1;
	%end;

	/************************/
	/* HDPS Integrity Check */
	/************************/
	%if (%upcase("&ANALYSIS.") eq "PS") %then %do ;
		%let message=;

		*Get groups that are not compared with a predefined model;
		data _NotPredefined;
			set infolder.&COMPARISON.(rename=comp=group where=(upcase(HDPS) ne "N")) 
			    infolder.&COMPARISON.(rename=control=group where=(upcase(HDPS) ne "N"));
			keep group HDPSWin: HDPS;
		run;

	   	proc means data=_NotPredefined nway noprint;
		    var HDPSWinfrom HDPSWinTo;
		    output out=HDPS_WIN min(HDPSWinfrom)= 
		                        max(HDPSWinTo)=;
		run;
			    
		data _NULL_;
			set HDPS_WIN;
			call symputx("MinHDPSWinfrom",put(HDPSWinfrom,best.));
			call symputx("MaxHDPSWinTo",put(HDPSWinTo,best.));
	    run;
		%put &MinHDPSWinfrom. &MaxHDPSWinTo.;

		proc sort nodupkey data=_NotPredefined;
			by group;
		run;

		*Create dataset to id which group for HDPS;
		data HDPSSettings;
			set _NotPredefined(in=b);
			by group;
			drop HDPSWin:;
		run;

		%ISDATA(dataset=infolder.&comparison.);	
		%IF %EVAL(&NOBS.>0) %THEN %DO;
				
			data HDPS;
				set infolder.&comparison.;
				where upcase(HDPS)="Y";
				format group $100.;
				if control ne "" then do;
					group=control;
					output;
				end;
				if comp ne "" then do;
					group=comp;
					output;
				end;
				keep group HDPSWinFrom HDPSWinTo;
			run;

			proc sort nodupkey data=HDPS;
				by group HDPSWinFrom  HDPSWinTo;
			run;

			*check if the same group is there more than once (then it is associated with mon than one HDPS window);
			proc means data=HDPS nway noprint;
				var  HDPSWinFrom  HDPSWinTo;
				class group;
				output out=HDPS min(HDPSWinFrom)=
				                 max(HDPSWinTo)=;
			run;

			%let HDPSPROB=0;
			data HDPS;
			set HDPS;
			if _FREQ_>1 then call symput("HDPSPROB","1");
			run;
			%put &HDPSPROB.;

			%IF %EVAL(&HDPSPROB. = 1) %THEN %DO;
				%put WARNING: The same group is associated with more than one HDPS Window;
			%END;
		%END;
	%end; /*End HDPS check*/

	%put NOTE: ******** END OF MACRO: ms_processinputfiles v2.7 ********;

%mend ms_processinputfiles;
