****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_create_figures.sas  
*
* Created (mm/dd/yyyy): 10/13/2017
* Last modified: 02/05/2019 
* Version: 2.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Create tables formatted to produce Figures for Type 5 report. Calculate KM curve
*
*  Program inputs:                                                                                   
*  	-
* 
*  Program outputs:                                                                                                                                       
*  	-
* 
*  PARAMETERS:        

*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   2.0		  1/2/2019	 AP			Modified for integration into QRP
* 
*   2.1		  2/5/2019	 AP			Added Figure 5
*
***************************************************************************************************;

%macro ms_t5_create_figures();

	%put =====> MACRO CALLED: ms_t5_create_figures v2.1;

	*counter for figure 5 groups;
	%let cnt = 0;

	/*** Loop through groups ***/
	%do order = 1 %to &num_allgroups.;

		*reset macro vars;
		%let runid = ;
		%let group = ;
		%let requester_defined_censor = ;
		%let type5file = ;
		%let output_figure5 = ;

		 data _null_;
			set &groupfile.(where=(order=&order.));
			call symputx("runid", runid);
			call symputx("group" , group);
			call symputx("agegroupfmt", agegroupfmt);
			call symputx("requester_defined_censor", requester_defined_censor);
			call symputx("type5file", type5file);
			call symputx("output_figure5",output_figure5);
			call symputx("exposure", exposure);
		run;

		/*** Figures 1, 2, and 3 ***/
		%if %sysfunc(prxmatch(m/F1|F2|F3/i,&tablelist.)) > 0 %then %do;

			data _tablestrata;
				set &tablefile.(where=(table in ('F1', 'F2', 'F3')));
				keep tablesub levelid:;
			run;
			proc sort data=_tablestrata nodupkey;
				by _all_;
			run;

			/*Loop through stratifications*/
			%isdata(dataset=_tablestrata);
			%let numstrat = &nobs.;

			%do strat = 1 %to &numstrat.;
				
				/*reset level macro var*/
				%let levelid = ;

				data _null_;
		 			set _tablestrata;
					if _n_ = &strat. then do;
						call symputx('levelid', levelid1);
						call symputx('tablesub', tablesub);
					end;
				run;

				%if "&tablesub" = "A" %then %let stratvars = ;
				%if "&tablesub" = "B" %then %let stratvars = sex ;
				%if "&tablesub" = "C" %then %let stratvars = agegroup agegroupnum;
				%if "&tablesub" = "D" %then %let stratvars = race;
				%if "&tablesub" = "E" %then %let stratvars = hispanic;

				proc sort data=agg_firsteps(where=(level="&levelid" and mntsfromstart >=0)) 
					out=figure123_&tablesub.(keep=mntsfromstart episodes npts daysupp adjustedcodecount &stratvars.);
					by &stratvars. mntsfromstart;
					where group = "&group." and runid = "&runid.";
				run;

				data figure123_&order._&tablesub.;
					set figure123_&tablesub.;
					by &stratvars. mntsfromstart;

					*Calculate cumulative values;
					%if "&stratvars." ne "" %then %do;
					if first.&stratvars. then do;
					%end;
					%if "&stratvars." = "" %then %do;
					if _n_ = 1 then do;
					%end;
						cumulative_npts = npts;
						cumulative_daysupp = daysupp;
						cumulative_adjustedcodecount = adjustedcodecount;
					end;
					else do;
						cumulative_npts = sum(cumulative_npts, npts);
						cumulative_daysupp = sum(cumulative_daysupp, daysupp);
						cumulative_adjustedcodecount = sum(cumulative_adjustedcodecount, adjustedcodecount);
					end;
					retain cumulative_npts cumulative_daysupp cumulative_adjustedcodecount;

					*assign label;
					%if "&stratvars." = "sex" %then %do;
						if &stratvars. = 'F' then label = 'Female';
						if &stratvars. = 'M' then label = 'Male'; 
						if &stratvars. = 'O' then label = 'Other';
					%end;
					%if "&stratvars." = "agegroup" %then %do;
						label = agegroup;
					%end;
					%if "&stratvars." = "race" %then %do;
						if &stratvars. = '0' then label = 'Unknown';
						if &stratvars. = '1' then label = 'American Indian/Alaska Native';
						if &stratvars. = '2' then label = 'Asian';
						if &stratvars. = '3' then label = 'Black/African American';
						if &stratvars. = '4' then label = 'Native Hawaiian/Other Pacific Islander';
						if &stratvars. = '5' then label = 'White';
						%end;
					%if "&stratvars." = "hispanic" %then %do;
						if &stratvars.= 'N' then label = 'Not Hispanic Origin';
						if &stratvars. = 'Y' then label = 'Hispanic Origin';
						if &stratvars. = 'U' then label = 'Unknown';
					%end;
				run;

				%macro wrapper();

					%let agegroup_orderformatted =;
					%if "&stratvars." = "agegroup" %then %do;
						*merge in agegroupfmt;
						%if "&agegroupfmt" ne "" %then %do;
							data figure_age_formats;
								set figure123_&order._&tablesub.(keep=agegroup);
								format formattedage $75.;
								formattedage = strip(put(agegroup, $age&order.fmt.));
							run;

							proc sql noprint;
								select distinct formattedage into: agegroup_orderformatted separated by '*'
								from figure_age_formats;
							quit;
						%end;
						%else %do;
							proc sql noprint;
								select distinct agegroup into: agegroup_orderformatted separated by '*'
								from figure123_&order._&tablesub.;
							quit;
						%end;
					%end;

					%if "&tablesub" ne "A" %then %do;
						%isdata(dataset=attrmap_&order);
						%if %eval(&nobs.<1) %then %do;
							data attrmap_&order;
								length linecolor $15. id $20. value $50.;
							    linethickness = 2;	
						%end;
						%else %do;
							data attrmap_&order;
								set attrmap_&order;
						%end;

								%if "&stratvars." = "sex" %then %do;
								id = "sex";
								%do clr = 1 %to 3;
									%let color = %scan(&colors, &clr);
									%let value = %scan(Female Male Other, &clr);
									value = "&value";
									linecolor = "&color";
									output;
					            %end;
								%end;

								%if "&stratvars." = "race" %then %do;
								id = "race";
								%do clr = 1 %to 6;
									%let value = %scan(&colors, &clr);
									%let cat = %scan(Unknown*American Indian/Alaska Native*Asian*Black/African American*Native Hawaiian/Other Pacific Islander*White, &clr, %str(*));
									value = "&cat";
									linecolor = "&value";
									output;
								%end;
								%end;

								%if "&stratvars." = "hispanic" %then %do;
								id = "hispanic";
								%do clr = 1 %to 3;
									%let value = %scan(&colors, &clr);
									%let cat = %scan(Hispanic (No)*Hispanic (Yes)*Hispanic (Unknown), &clr, %str(*));
									value = "&cat";
									linecolor = "&value";
									output;
								%end;
								%end;
							
								%if "&stratvars." = "agegroup" %then %do;
								id = "agegroup";
				            	%do clr=1 %to &&agegroup_&grp.count.;
									%let color = %scan(&colors, &clr);
							        %let value=%scan(&&agegroup_&order.formatted, &clr, %str(*));
									value = "&value";
									linecolor = "&color";
									output;
				                %end;
								%end;
						run;
					%end;
				%mend;
				%wrapper();

			%end; /*loop through stratifications*/

		%end; /*Figures 1, 2, 3*/

		/*** Figures 4 and 5 ***/
		%if %sysfunc(prxmatch(m/F4|F5/i,&tablelist.)) > 0 %then %do;
	
			*Get levelid;
			data _null_;
	 			set &tablefile.(where=(table in ("F4", "F5")));
				if _n_ =1 then do;
				call symputx('levelid', levelid1);
				end;
			run;

			%let cens_dth = ;
			%let cens_qryend = ;
			%let cens_dpend = ;
			%let cens_spec = ;
			%let censor_dth=;
			%let censor_qryend=;
			%let censor_dpend=;

			data _null_;
		  		set infolder.&type5file.(where=(lowcase(group)="&group"));
				  call symputx("censor_dth", upcase(censor_dth));
				  call symputx("censor_qryend", upcase(censor_qryend));
				  call symputx("censor_dpend", upcase(censor_dpend));
			run;

			%if "&censor_dth" = "Y" %then %do; %let cens_dth = cens_dth; %end;
			%if "&censor_qryend" = "Y" %then %do; %let cens_qryend = cens_qryend; %end;
			%if "&censor_dpend" = "Y" %then %do; %let cens_dpend = cens_dpend; %end;

			%if %length(&requester_defined_censor) > 0 & &requester_defined_censor ne %str() %then %do; %let cens_spec = cens_spec; %end;

			*list of censoring criteria;
			%global censor_list&order. censor_cnt&order.;
			%let censor_list&order = cens_elig cens_episend &cens_dth. &cens_qryend. &cens_dpend. &cens_spec.;
			%put &&censor_list&order.;

			%let censor_cnt&order = %sysfunc(countw(&&censor_list&order, %str( )));
			%put &&censor_cnt&order;

			*create variable ists for arrays;
			%let cumulative_list=;
			%let sum_list=;

			%do x =1 %to &&censor_cnt&order.;
				%let cumulative_list = &cumulative_list. %sysfunc(cats(cumulative_,%quote(%scan(&&censor_list&order., &x.))));
				%let sum_list = &sum_list. %sysfunc(cats(sum_,%quote(%scan(&&censor_list&order., &x.))));
			%end;

			/*Labels for censoring criteria*/
			%let cens_elig_label = Disenrollment;
			%let cens_dth_label = Evidence of Death;
			%let cens_dpend_label =End of Data Partner Data;
			%let cens_qryend_label =End of Query Period;
			%let cens_episend_label =End of Exposure Episode;
			%let cens_spec_label =&requester_defined_censor.;

			*** Calculate KM curve **;

			*aggregate all sites;
			proc means data=agg_episdur_censor(where=(group="&group" and runid = "&runid.")) noprint nway;
				var episodes &&censor_list&order.;
				class group episodelength / missing;
				output out=_survival_tmp(drop=_:) sum=;
				where level = "&levelid." and episodenum = 1;
			run;

			proc sort data=_survival_tmp;
				by group episodelength;
			run;

			*Calculate total number of episodes for each censoring criteria (cens_cum_list);
			data _cumulative_totals(keep=group &cumulative_list. cumulative_episodes);
				array cens_cumulative_list{*} &cumulative_list. cumulative_episodes;
				array censor {*} &&censor_list&order. episodes;

				do i = 1 to dim(cens_cumulative_list);
					cens_cumulative_list{i} = 0;
				end;

				do until(last.group);
					set _survival_tmp;  
					by group;

					do a = 1 to dim(cens_cumulative_list);
						cens_cumulative_list{a} = cens_cumulative_list{a} + censor{a};
					end;
				end;
			run;

			*Add in 0 censdays_value ;
			data _survival_tmp;
				set _survival_tmp;
				by group;

				if first.group then do;
					output;
					array censor {*} &&censor_list&order. episodes;
						do i = 1 to dim(censor);
							censor{i} = 0;
						end;

					episodelength = 0;
					output;
				end;
				else do;
					output;
				end;
				drop i;
			run;

			proc sort data=_survival_tmp;
				by group episodelength;
			run;

			*KM curve;
			data Figure4_&order.;
				set _survival_tmp; 
				by group episodelength;

				array censor_sum{*} &sum_list. sum_episodes;
				array censor{*} &&censor_list&order. episodes;
				array cens_cumulative_list{*} &cumulative_list.; 

				if first.group then do;
					merge _cumulative_totals;
					by group;
			
					do i = 1 to dim(censor);
						censor_sum{i} = censor{i};
					end;
				end;
				else do;
					do i = 1 to dim(censor);
						censor_sum{i} = censor{i} + censor_sum{i} ;
					end;
				end;

				retain &sum_list. sum_episodes;

				/*-----KM Plot--------------------*/
				episodes_left = cumulative_episodes - sum_episodes;
				lag_episodes_left = lag(episodes_left);

				if first.group then do;
					lag_episodes_left = .; 
					%do cns = 1 %to &&censor_cnt&order.;
						km_%scan(&&censor_list&order., &cns.) = 1;
					%end;
				end;
				else do;
					%do cns = 1 %to &&censor_cnt&order.;
						if %scan(&&censor_list&order., &cns.) > 0 then do;
							km_%scan(&&censor_list&order., &cns.) = km_%scan(&&censor_list&order., &cns.)*(1-(%scan(&&censor_list&order., &cns.)/lag_episodes_left));
						end;
						else do;
							km_%scan(&&censor_list&order., &cns.) = km_%scan(&&censor_list&order., &cns.);
						end;

					%end;
				end;
				retain %do cns = 1 %to &&censor_cnt&order.; km_%scan(&&censor_list&order., &cns.) %end; ;

				/*Assign label to censoring reason for use in figure*/
				%do lbl = 1 %to &&censor_cnt&order.;
					%let label = %scan(&&censor_list&order, &lbl.)_label;
					label %sysfunc(cats(km_,%quote(%scan(&&censor_list&order, &lbl.)))) = "&&&label.";
				%end;
			run;


			*Figure 5;
			%if %sysfunc(prxmatch(m/F5/i,&tablelist.)) > 0 %then %do;

				%let cens_spec_label = Occurence of request-defined censoring criteria;

				%if %length(&censoring_label_F5) > 0 & &censoring_label_F5 ne %str() %then %do;
					%let figure5label =&censoring_label_F5.;
				%end;
				%else %do;
					%let figure5label = &&&censoring_display_F5._label;
				%end;

				/*if output_figure5 = Y then add to figure5 dataset*/
				%if "&output_figure5." = "Y" %then %do;

					%let cnt = %eval(&cnt.+1);

					%if %eval(&cnt.) = 1 %then %do;
						data figure5;
							set Figure4_&order.(rename=km_&censoring_display_F5. = km_&cnt._&censoring_display_F5. keep=episodelength keep=km_&censoring_display_F5.);
							label km_&cnt._&censoring_display_F5. = "&exposure.";
						run;
					%end;
					%else %do;
						data figure5;
							merge figure5
								Figure4_&order.(rename=km_&censoring_display_F5. = km_&cnt._&censoring_display_F5. keep=episodelength keep=km_&censoring_display_F5.);
								by episodelength;
							label km_&cnt._&censoring_display_F5. = "&exposure.";
						run;
					%end;
				%end;

			%end; /*figure 5*/

		%end; /*Figure 4/5*/

	%end; /*loop through groups*/

	%put =====> END MACRO: ms_t5_create_figures v2.1;

%mend ms_t5_create_figures;
